@@ -1,6 +1,6 @@
 /* spu.c -- Assembler for the IBM Synergistic Processing Unit (SPU)
 
-   Copyright 2006, 2007 Free Software Foundation, Inc.
+   Copyright 2006, 2007, 2008, 2009 Free Software Foundation, Inc.
 
    This file is part of GAS, the GNU Assembler.
 
@@ -53,6 +53,7 @@ static const char *get_reg (const char *param, struct spu_insn *insn, int arg,
 			    int accept_expr);
 static int calcop (struct spu_opcode *format, const char *param,
 		   struct spu_insn *insn);
+static void spu_brinfo (int);
 static void spu_cons (int);
 
 extern char *myname;
@@ -82,6 +83,7 @@ const char FLT_CHARS[] = "dDfF";
 const pseudo_typeS md_pseudo_table[] =
 {
   {"align", s_align_ptwo, 4},
+  {"brinfo", spu_brinfo, 0},
   {"bss", s_lcomm_bytes, 1},
   {"def", s_set, 0},
   {"dfloat", float_cons, 'd'},
@@ -99,11 +101,14 @@ const pseudo_typeS md_pseudo_table[] =
   /* Likewise for eqv.  */
   {"eqv", NULL, 0},
   {".eqv", s_set, -1},
-  {"file", (void (*) PARAMS ((int))) dwarf2_directive_file, 0 }, 
+  {"file", (void (*) (int)) dwarf2_directive_file, 0 }, 
   {"loc", dwarf2_directive_loc, 0}, 
   {0,0,0}
 };
 
+/* Bits plugged into branch instruction offset field.  */
+unsigned int brinfo;
+
 void
 md_begin (void)
 {
@@ -120,7 +125,8 @@ md_begin (void)
     {
       /* hash each mnemonic and record its position */
 
-      retval = hash_insert (op_hash, spu_opcodes[i].mnemonic, (PTR)&spu_opcodes[i]);
+      retval = hash_insert (op_hash, spu_opcodes[i].mnemonic,
+			    (void *) &spu_opcodes[i]);
 
       if (retval != NULL && strcmp (retval, "exists") != 0)
 	as_fatal (_("Can't hash instruction '%s':%s"),
@@ -267,7 +273,7 @@ md_assemble (char *op)
   struct spu_insn insn;
   int i;
 
-  assert (op);
+  gas_assert (op);
 
   /* skip over instruction to find parameters */
 
@@ -341,6 +347,16 @@ md_assemble (char *op)
       as_warn (_("Treating '%-*s' as a symbol."), (int)(syntax_error_param - d), d);
     }
 
+  if (brinfo != 0
+      && (insn.tag <= M_BRASL
+	  || (insn.tag >= M_BRZ && insn.tag <= M_BRHNZ))
+      && (insn.opcode & 0x7ff80) == 0
+      && (insn.reloc_arg[0] == A_R18
+	  || insn.reloc_arg[0] == A_S18
+	  || insn.reloc_arg[1] == A_R18
+	  || insn.reloc_arg[1] == A_S18))
+    insn.opcode |= brinfo << 7;
+
   /* grow the current frag and plop in the opcode */
 
   thisfrag = frag_more (4);
@@ -369,6 +385,9 @@ md_assemble (char *op)
 	fixP->tc_fix_data.insn_tag = insn.tag;
       }
   dwarf2_emit_insn (4);
+
+  /* .brinfo lasts exactly one instruction.  */
+  brinfo = 0;
 }
 
 static int
@@ -751,6 +770,39 @@ md_create_long_jump (char *ptr,
 }
 #endif
 
+/* Handle .brinfo <priority>,<lrlive>.  */
+static void
+spu_brinfo (int ignore ATTRIBUTE_UNUSED)
+{
+  addressT priority;
+  addressT lrlive;
+
+  priority = get_absolute_expression ();
+  SKIP_WHITESPACE ();
+
+  lrlive = 0;
+  if (*input_line_pointer == ',')
+    {
+      ++input_line_pointer;
+      lrlive = get_absolute_expression ();
+    }
+
+  if (priority > 0x1fff)
+    {
+      as_bad (_("invalid priority '%lu'"), (unsigned long) priority);
+      priority = 0;
+    }
+
+  if (lrlive > 7)
+    {
+      as_bad (_("invalid lrlive '%lu'"), (unsigned long) lrlive);
+      lrlive = 0;
+    }
+
+  brinfo = (lrlive << 13) | priority;
+  demand_empty_rest_of_line ();
+}
+
 /* Support @ppu on symbols referenced in .int/.long/.word/.quad.  */
 static void
 spu_cons (int nbytes)
@@ -897,6 +949,7 @@ void
 md_apply_fix (fixS *fixP, valueT *valP, segT seg ATTRIBUTE_UNUSED)
 {
   unsigned int res;
+  unsigned int mask;
   valueT val = *valP;
   char *place = fixP->fx_where + fixP->fx_frag->fr_literal;
 
@@ -936,93 +989,114 @@ md_apply_fix (fixS *fixP, valueT *valP, segT seg ATTRIBUTE_UNUSED)
   fixP->fx_addnumber = val;
 
   if (fixP->fx_r_type == BFD_RELOC_SPU_PPU32
-      || fixP->fx_r_type == BFD_RELOC_SPU_PPU64)
+      || fixP->fx_r_type == BFD_RELOC_SPU_PPU64
+      || fixP->fx_r_type == BFD_RELOC_SPU_ADD_PIC)
     return;
 
   if (fixP->fx_addsy == NULL && fixP->fx_pcrel == 0)
     {
       fixP->fx_done = 1;
       res = 0;
+      mask = 0;
       if (fixP->tc_fix_data.arg_format > A_P)
 	{
 	  int hi = arg_encode[fixP->tc_fix_data.arg_format].hi;
 	  int lo = arg_encode[fixP->tc_fix_data.arg_format].lo;
 	  if (hi > lo && ((offsetT) val < lo || (offsetT) val > hi))
 	    as_bad_where (fixP->fx_file, fixP->fx_line,
-			  "Relocation doesn't fit. (relocation value = 0x%lx)",
+			  _("Relocation doesn't fit. (relocation value = 0x%lx)"),
 			  (long) val);
 	}
 
       switch (fixP->fx_r_type)
-        {
-        case BFD_RELOC_8:
+	{
+	case BFD_RELOC_8:
 	  md_number_to_chars (place, val, 1);
 	  return;
 
-        case BFD_RELOC_16:
+	case BFD_RELOC_16:
 	  md_number_to_chars (place, val, 2);
 	  return;
 
-        case BFD_RELOC_32:
+	case BFD_RELOC_32:
+	case BFD_RELOC_32_PCREL:
 	  md_number_to_chars (place, val, 4);
 	  return;
 
-        case BFD_RELOC_64:
+	case BFD_RELOC_64:
 	  md_number_to_chars (place, val, 8);
 	  return;
 
-        case BFD_RELOC_SPU_IMM7:
-          res = (val & 0x7f) << 14;
-          break;
+	case BFD_RELOC_SPU_IMM7:
+	  res = val << 14;
+	  mask = 0x7f << 14;
+	  break;
 
-        case BFD_RELOC_SPU_IMM8:
-          res = (val & 0xff) << 14;
-          break;
+	case BFD_RELOC_SPU_IMM8:
+	  res = val << 14;
+	  mask = 0xff << 14;
+	  break;
 
-        case BFD_RELOC_SPU_IMM10:
-          res = (val & 0x3ff) << 14;
-          break;
+	case BFD_RELOC_SPU_IMM10:
+	  res = val << 14;
+	  mask = 0x3ff << 14;
+	  break;
 
-        case BFD_RELOC_SPU_IMM10W:
-          res = (val & 0x3ff0) << 10;
-          break;
+	case BFD_RELOC_SPU_IMM10W:
+	  res = val << 10;
+	  mask = 0x3ff0 << 10;
+	  break;
 
-        case BFD_RELOC_SPU_IMM16:
-          res = (val & 0xffff) << 7;
-          break;
+	case BFD_RELOC_SPU_IMM16:
+	  res = val << 7;
+	  mask = 0xffff << 7;
+	  break;
 
-        case BFD_RELOC_SPU_IMM16W:
-          res = (val & 0x3fffc) << 5;
-          break;
+	case BFD_RELOC_SPU_IMM16W:
+	  res = val << 5;
+	  mask = 0x3fffc << 5;
+	  break;
 
-        case BFD_RELOC_SPU_IMM18:
-          res = (val & 0x3ffff) << 7;
-          break;
+	case BFD_RELOC_SPU_IMM18:
+	  res = val << 7;
+	  mask = 0x3ffff << 7;
+	  break;
 
-        case BFD_RELOC_SPU_PCREL9a:
-          res = ((val & 0x1fc) >> 2) | ((val & 0x600) << 14);
-          break;
+	case BFD_RELOC_SPU_PCREL9a:
+	  res = ((val & 0x1fc) >> 2) | ((val & 0x600) << 14);
+	  mask = (0x1fc >> 2) | (0x600 << 14);
+	  break;
 
-        case BFD_RELOC_SPU_PCREL9b:
-          res = ((val & 0x1fc) >> 2) | ((val & 0x600) << 5);
-          break;
+	case BFD_RELOC_SPU_PCREL9b:
+	  res = ((val & 0x1fc) >> 2) | ((val & 0x600) << 5);
+	  mask = (0x1fc >> 2) | (0x600 << 5);
+	  break;
 
-        case BFD_RELOC_SPU_PCREL16:
-          res = (val & 0x3fffc) << 5;
-          break;
+	case BFD_RELOC_SPU_PCREL16:
+	  res = val << 5;
+	  mask = 0x3fffc << 5;
+	  break;
 
-        default:
-          as_bad_where (fixP->fx_file, fixP->fx_line,
-                        _("reloc %d not supported by object file format"),
-                        (int) fixP->fx_r_type);
-        }
+	case BFD_RELOC_SPU_HI16:
+	  res = val >> 9;
+	  mask = 0xffff << 7;
+	  break;
 
-      if (res != 0)
-        {
-          place[0] |= (res >> 24) & 0xff;
-          place[1] |= (res >> 16) & 0xff;
-          place[2] |= (res >> 8) & 0xff;
-          place[3] |= (res) & 0xff;
-        }
+	case BFD_RELOC_SPU_LO16:
+	  res = val << 7;
+	  mask = 0xffff << 7;
+	  break;
+
+	default:
+	  as_bad_where (fixP->fx_file, fixP->fx_line,
+			_("reloc %d not supported by object file format"),
+			(int) fixP->fx_r_type);
+	}
+
+      res &= mask;
+      place[0] = (place[0] & (~mask >> 24)) | ((res >> 24) & 0xff);
+      place[1] = (place[1] & (~mask >> 16)) | ((res >> 16) & 0xff);
+      place[2] = (place[2] & (~mask >> 8)) | ((res >> 8) & 0xff);
+      place[3] = (place[3] & ~mask) | (res & 0xff);
     }
 }
